﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using System.Threading.Tasks;

using TestProject.Models;

namespace TestProject.Pages
{
    /// <summary>
    /// 항목 모델
    /// </summary>
    public class ItemModel : PageModel
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 할일 클라이언트
        /// </summary>
        private readonly TodoClient client;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 할일 입력 - TodoInput

        /// <summary>
        /// 할일 입력
        /// </summary>
        [BindProperty]
        public TodoInputModel TodoInput { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ItemModel(client)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="client">할일 클라이언트</param>
        public ItemModel(TodoClient client)
        {
            this.client = client;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region GET 요청시 처리하기 (비동기) - OnGetAsync(id)

        /// <summary>
        /// GET 요청시 처리하기 (비동기)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> OnGetAsync(int id)
        {
            TodoModel todo = await this.client.GetItemAsync(id);

            if(todo == null)
            {
                return RedirectToPage("/Index");
            }

            TodoInput = new TodoInputModel
            {
                Name       = todo.Name,
                IsComplete = todo.IsComplete
            };

            return Page();
        }

        #endregion
        #region POST 저장 요청시 처리하기 (비동기) - OnPostSaveAsync(id)

        /// <summary>
        /// POST 저장 요청시 처리하기 (비동기)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> OnPostSaveAsync(int id)
        {
            if(!ModelState.IsValid)
            {
                return Page();
            }

            TodoModel updateTodo = new TodoModel
            {
                ID         = id,
                Name       = TodoInput.Name,
                IsComplete = TodoInput.IsComplete
            };

            await this.client.SaveItemAsync(updateTodo);

            return RedirectToPage("/Index");
        }

        #endregion
        #region POST 삭제 요청시 처리하기 (비동기) - OnPostDeleteAsync(id)

        /// <summary>
        /// POST 삭제 요청시 처리하기 (비동기)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> OnPostDeleteAsync(int id)
        {
            await this.client.DeleteItemAsync(id);

            return RedirectToPage("/Index");
        }

        #endregion
    }
}